/*
 * ACD Applet
 * v1.0 8/97
 *
 * This applet demonstrates the use of the  Jtapi Call Center package.
 * It gets the ACDAddresses known to the provider, and the agents logged-in
 * at those ACD splits.  It tries to log-ib two agents specified as
 * parameters in the acd.html file.
 *
 */


import java.applet.*;
import java.awt.*;

import javax.telephony.*;
import javax.telephony.events.AddrEv;
import javax.telephony.callcenter.*;
import javax.telephony.callcenter.events.*;

import com.lucent.jtapi.tsapi.*;

public class ACD extends Applet
{
    String          url;
    String          acd;
    String          agent1 = null;
    String          agent2 = null;
    JtapiPeer       myPeer;
    Provider        myProvider;
    Address         myAddress;
    ACDAddress      myACDAddress;
    MyACDObserver   myACDObserver;
    Agent           myAgent1;
    Agent           myAgent2;
    AgentTerminal   myAgent1Terminal = null;
    AgentTerminal   myAgent2Terminal = null;
    Address         myAgent1Address = null;
    Address         myAgent2Address = null;
	MyTextArea      trace;

    public void init()
    {
        String          tlink = null;
		String          user = null;
	    String          password = null;
	    String          callingDev = null;
	    String          path = null;
        
        //{{INIT_CONTROLS
		setLayout(null);
		addNotify();
		resize(350,170);
		trace = new MyTextArea();
		trace.reshape(0,0,350,150);
		add(trace);
		//}}

		// get tlink parameter from html file
		path = getParameter("PATH");

        // get tlink parameter from html file
		tlink = getParameter("TLINK");
        if (tlink != null) 
        {
            tlink.toUpperCase();
        }

		// get user parameter from html file
	    user = getParameter("USER");

	    // get user password from html file
	    password = getParameter("PASSWORD");

		// get calling device from html file
	    acd = getParameter("ACD");

	    // get calling device from html file
	    agent1 = getParameter("AGENT1");

	    // get calling device from html file
	    agent2 = getParameter("AGENT2");

        url = tlink + ";loginID=" + user + ";passwd=" + password;

        try
        {
            myPeer = JtapiPeerFactory.getJtapiPeer(path);
            myProvider = myPeer.getProvider(url);
            if (myProvider != null) 
            {
                while (myProvider.getState() != Provider.IN_SERVICE)
                {
                    try
                    {
        		        Thread.sleep(3000);
        		    }
        		    catch(InterruptedException e) {}
                }
                if (myProvider.getState() == Provider.IN_SERVICE)
                {
                    trace.appendText("Provider is IN_SERVICE\n\n");
                }
            } 
            else 
            {
                trace.appendText("Unable to create provider, verify that link is up and\n" +
                                  "that the Tsapi.pro file has the Tserver's TCP/IP address\n\n");
            }
        }
        catch (Exception e)
        {
            trace.appendText( "Exception caught. Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
    }
    
    public void start()
    {
        ACDAddress[]    acdAddresses = null;
        Agent[]         agents = null;
        
        if (myProvider != null)
        {
            if (myProvider instanceof CallCenterProvider)
            {
    	        try
    	        {
    	            acdAddresses = ( (CallCenterProvider) myProvider).getACDAddresses();
    	        }
                catch(MethodNotSupportedException e)
                {
                    trace.appendText ( "ACDAddress.getLoggedOnAgents(): caught MethodNotSupportedException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
        
                if (acdAddresses != null) 
                {
    	            trace.appendText("ACDAddresses for this provider:" + "\n");
        	        for(int i=0; i<acdAddresses.length; i++)
        	        {
        	            trace.appendText("\tACDAddress=" + acdAddresses[i].getName() + "\n");
        	        }
        	        trace.appendText("\n");

        	        for(int i=0; i<acdAddresses.length; i++)
                    {
                       try
                       {
                            agents = acdAddresses[i].getLoggedOnAgents();
                       }
                       catch(Exception e)
                       {
                            trace.appendText( "Exception caught - Message: " + e.getMessage() + "\n\n" );
                	        e.printStackTrace();
                       }
                       if (agents != null) 
                       {
                           trace.appendText("Agents at " +  acdAddresses[i].getName() + ":\n");
                           for(int k=0; k<agents.length; k++)
                           {
                                trace.appendText("\tAgent[" + k + "]=" + agents[k].getAgentTerminal().getName() + "\n");
                           }
                           trace.appendText("\n");
                       } 
                       else 
                       {
                            trace.appendText("Agents at " +  acdAddresses[i].getName() + ": <none>\n\n");
                       }
        	        }
        	    } 
        	    else 
        	    {
        	        trace.appendText("ACDAddresses for this provider: <none>\n\n");
        	    }
            }

            try 
            {
                myAddress = myProvider.getAddress(acd);
            }
            catch (InvalidArgumentException e) 
            {
    	        trace.appendText ( "Provider.getAddress(): caught InvalidArgumentException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            
            if (myAddress != null) 
            {
                //if Address returned does not implement ACDAddress return
                if (!(myAddress instanceof ACDAddress)) 
                {
                    trace.appendText("Address" + myAddress.getName() + "doesn't implement ACDAddress\n\n");
                    return;
                }
            } 
            else 
            {
                trace.appendText("Could not get address\n\n");
                return;
            }

            myACDAddress = (ACDAddress)myAddress;
            
            agents = null;
            try
            {
                agents = myACDAddress.getLoggedOnAgents();
            }
            catch(MethodNotSupportedException e)
            {
                trace.appendText ( "ACDAddress.getLoggedOnAgents(): caught MethodNotSupportedException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }

            trace.appendText("Agents/States currently at " +  myACDAddress.getName() + ":\n");
            if (agents != null) 
            {
                for(int i=0; i<agents.length; i++)
                {
                    trace.appendText("\tAgent[" + i + "]=" + agents[i].getAgentTerminal().getName() +
                                     " State: " + stateToString(agents[i].getState()) + "\n");
                }
                trace.appendText("\n");
            }
            else 
            {
                trace.appendText("Agents is null\n");
            }
            
            myACDObserver = new MyACDObserver(trace);
            // add observer on my ACD Address
            try
            {
                myAddress.addObserver(myACDObserver);
            }
            catch(Exception e)
            {
                trace.appendText( "addObserver() caught Exception - Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }

            trace.appendText("Logging agents: " + agent1 + ", " + agent2 + " to ACD " + myAddress.getName() + "...\n\n");

            try 
            {
                myAgent1Address = myProvider.getAddress(agent1);
                myAgent2Address = myProvider.getAddress(agent2);
                myAgent1Terminal = (AgentTerminal)myProvider.getTerminal(agent1);
                myAgent2Terminal = (AgentTerminal)myProvider.getTerminal(agent2);
            }
            catch (InvalidArgumentException e) 
            {
    	        trace.appendText ( "Provider.getAddress(): caught InvalidArgumentException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            catch (PlatformException e) 
            {
    	        trace.appendText ( "Provider.getAddress(): caught PlatformException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }

            if (myAgent1Terminal != null) 
            {
                try 
                {

                    myAgent1 = myAgent1Terminal.addAgent(myAgent1Address,
                                                       myACDAddress,
                                                       Agent.LOG_IN,
                                                       agent1,
                                                       "");

                }
                catch (InvalidStateException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent1 + "): caught InvalidStateException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (ResourceUnavailableException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent1 + "): caught ResourceUnavailableException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (InvalidArgumentException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent1 + "): caught InvalidArgumentException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (PlatformException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent1 + "): caught PlatformException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
            } 
            else 
            {
                trace.appendText( "Agent terminal for " + agent1 + "is null.\n\n" );
            }
            
            if (myAgent2Terminal != null) 
            {
                try 
                {

                    myAgent2 = myAgent2Terminal.addAgent(myAgent2Address,
                                                       myACDAddress,
                                                       Agent.LOG_IN,
                                                       agent2,
                                                       "");
                }
                catch (InvalidStateException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent2 + "): caught InvalidStateException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (ResourceUnavailableException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent2 + "): caught ResourceUnavailableException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (InvalidArgumentException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent2 + "): caught InvalidArgumentException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
                catch (PlatformException e) 
                {
        	        trace.appendText ( "TerminalAgent.addAgent(" + agent2 + "): caught PlatformException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
            } 
            else 
            {
                trace.appendText( "Agent terminal for " + agent2 + "is null.\n\n" );
            }
            
            if (myACDAddress != null) 
            {
                try
                {
                    agents = myACDAddress.getLoggedOnAgents();
                }
                catch(MethodNotSupportedException e)
                {
                    trace.appendText ( "ACDAddress.getLoggedOnAgents(): caught MethodNotSupportedException\n" );
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }

                trace.appendText("Agents/State/WorkMode currently at " +  myACDAddress.getName() + ":\n");
                if (agents != null) 
                {
                    for(int i=0; i<agents.length; i++)
                    {
                        LucentAgentStateInfo stateInfo = ( (LucentAgent)agents[i]).getStateInfo();
                        
                        if ( (stateInfo.state != Agent.LOG_OUT) && 
                             (stateInfo.workMode != LucentAgent.MODE_AUTO_IN) ) 
                        {
                            try 
                            {
                                ( (LucentAgent)agents[i]).setState(Agent.READY, LucentAgent.MODE_AUTO_IN);
                            }
                            catch (TsapiInvalidStateException e) 
                            {
                    	        trace.appendText ( "LucentAgent.setState(): caught TsapiInvalidStateException\n" );
                                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
                    	        e.printStackTrace();
                            }
                            catch (TsapiInvalidArgumentException e) 
                            {
                    	        trace.appendText ( "LucentAgent.setState(): caught TsapiInvalidArgumentException\n" );
                                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
                    	        e.printStackTrace();
                            }
                        }
                        stateInfo = ( (LucentAgent)agents[i]).getStateInfo();
                        trace.appendText("\tAgent[" + i + "]=" + agents[i].getAgentTerminal().getName() +
                                         "\tState: " + stateToString(stateInfo.state) +
                                         "\tWorkMode: " + modeToString(stateInfo.workMode) + "\n");
                     }
                }
                trace.appendText("\n");
            } 
            else 
            {
                trace.appendText( "ACD address for " + acd + "is null.\n\n" );
            }
        }
    }

    public String stateToString(int state) {

        String strState;

        switch (state)
        {
            case (Agent.LOG_IN):
                strState = "LOG_IN";
                break;
            case (Agent.LOG_OUT):
                strState = "LOG_OUT";
                break;
            case (Agent.NOT_READY):
                strState = "NOT_READY";
                break;
            case (Agent.READY):
                strState = "READY";
                break;
            case (Agent.WORK_NOT_READY):
                strState = "WORK_NOT_READY";
                break;
            case (Agent.WORK_READY):
                strState = "WORK_READY";
                break;
            case (Agent.BUSY):
                strState = "BUSY";
                break;
            case (Agent.UNKNOWN):
                strState = "UNKNOWN";
                break;
            default:
                strState = "Other: " + state;
                break;
        }
        return strState;
    }

    public static String modeToString(int mode) {

        String strMode;

        switch (mode)
        {
            case LucentAgent.MODE_AUTO_IN :
                strMode = "MODE_AUTO_IN";
                break;
            case LucentAgent.MODE_MANUAL_IN :
                strMode = "MODE_MANUAL_IN";
                break;
            case LucentAgent.MODE_NONE :
                strMode = "MODE_NONE";
                break;
            default:
                strMode = "Other: " + mode;
                break;
        }
        return strMode;
    }

    public void stop()
    {
        if (myACDAddress != null)
        {
            // delete observer on my ACD Address
            try
            {
                myAddress.removeObserver(myACDObserver);
                myProvider.shutdown();
            }
            catch (Exception e)
            {
                    trace.appendText("Exception occured\n");
                    trace.appendText( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                    return;
            }
        }
    }

    public void paint( Graphics d ) {
    	Dimension size;

    	size = size();
    	trace.resize( size.width, size.height - 20);
    	return;
    }


	//{{DECLARE_CONTROLS
	//}}
}

class MyACDObserver implements ACDAddressObserver
{
    MyACDObserver() {
    }

    MyACDObserver ( MyTextArea traceArea ) {
        this();
        trace = traceArea;
    }

    public void addressChangedEvent(AddrEv[] event)
    {
        for(int i=0; i<event.length; i++)
        {
            if(!(event[i] instanceof ACDAddrEv))
                return;
            ACDAddrEv aaEvent = (ACDAddrEv)event[i];

            switch(aaEvent.getID())
            {
                case ACDAddrBusyEv.ID:
                    {
                        trace.appendText("Received ACD Addr Busy Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrLoggedOffEv.ID:
                    {
                        trace.appendText("Received ACD Addr Logged Off Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrLoggedOnEv.ID:
                    {
                        trace.appendText("Received ACD Addr Logged On Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrNotReadyEv.ID:
                    {
                        trace.appendText("Received ACD Not Ready Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrReadyEv.ID:
                    {
                        trace.appendText("Received ACD Addr Ready Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrUnknownEv.ID:
                    {
                        trace.appendText("Received ACD Addr Unknown Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrWorkNotReadyEv.ID:
                    {
                        trace.appendText("Received ACD Addr Work Not Ready Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                case ACDAddrWorkReadyEv.ID:
                    {
                        trace.appendText("Received ACD Addr Work Ready Ev - Agent: " + aaEvent.getAgent().getAgentTerminal().getName() + 
                                         " Cause: " + aaEvent.getCallCenterCause() + "\n");
                        break;
                    }
                default:
                        break;
            }
        }
    }
    MyTextArea trace;
}

