/*
 * Route Applet
 * v1.0 8/97
 *
 * This applet demonstrates the use of the Jtapi Call Center package.
 * It is a routing application, it registers the VDN specified in
 * the route.html for routing.  When a call is received by the VDN,
 * it simply routes it to the destination specified in route.html.
 *
 * Note that an example of the switch administration for routing is:
 * 
 * VDN: 3003 Vector: 300
 * Vector: 300 
 *      Step 1: adjunct routing to link 50001 (ASAI station)
 *      Step 2: wait 30 seconds of ringback
 *      Step 3: stop
 *
 */

import java.applet.*;
import java.awt.*;

import javax.telephony.*;
import javax.telephony.callcenter.*;
import javax.telephony.callcenter.events.*;
import com.lucent.jtapi.tsapi.*;

public class Route extends Applet
{
    String      vdn;
    String      destination;

    JtapiPeer   myPeer;
    Provider    myProvider;

    RouteAddress    myRouteAddress;
    MyRouteCallback myRouteCallback;
	MyTextArea      trace;
    
    public void init()
    {
	    String          url = null;
		String          tlink = null;
		String          user = null;
	    String          password = null;
	    String          path = null;

        //{{INIT_CONTROLS
		setLayout(null);
		addNotify();
		resize(350,170);
		trace = new MyTextArea();
		trace.reshape(0,0,350,150);
		add(trace);
		//}}
		
		// get tlink parameter from html file
		path = getParameter("PATH");

        // get tlink parameter from html file
		tlink = getParameter("TLINK");
        if (tlink != null) {
            tlink.toUpperCase();
        }

		// get user parameter from html file
	    user = getParameter("USER");

	    // get user password from html file
	    password = getParameter("PASSWORD");

		// get vdn parameter
	    vdn = getParameter("VDN");

	    // get destination parameter
	    destination = getParameter("DESTINATION");

        url = tlink + ";loginID=" + user + ";passwd=" + password;

        try
        {
            myPeer = JtapiPeerFactory.getJtapiPeer(path);
            myProvider = myPeer.getProvider(url);
            if (myProvider != null)
            {
                while (myProvider.getState() != Provider.IN_SERVICE)
                {
                    try
                    {
        		        Thread.sleep(3000);
        		    }
        		    catch(InterruptedException e) {}
                }
                if (myProvider.getState() == Provider.IN_SERVICE)
                {
                    trace.appendText("Provider is IN_SERVICE\n");
                }
            } 
            else 
            {
                trace.appendText("Unable to create provider, verify that link is up and\n" +
                                  "that the Tsapi.pro file has the Tserver's TCP/IP address\n\n");
            }
        }
        catch (Exception e)
        {
            trace.appendText( "Exception caught - Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
   }

   public void start()
   {
        Address     myAddress;
        
        if (myProvider != null)
        {
            try
            {
                myAddress = myProvider.getAddress(vdn);
                trace.appendText("Address " + myAddress.getName() + " created\n");
                if (!(myAddress instanceof RouteAddress)) {
                    trace.appendText("Address " + myAddress.getName() + " is not instance of RouteAddress\n");
                    return;
                }
                trace.appendText("Address " + myAddress.getName() + " is instance of RouteAddress\n");
                myRouteAddress = (RouteAddress)myAddress;
                myRouteCallback = new MyRouteCallback(destination, trace);
                
                // register callback to route calls for myRouteAddress
                myRouteAddress.registerRouteCallback(myRouteCallback);
                trace.appendText("Registered Route Callback with " + myRouteAddress.getName() + "\n");
                trace.appendText("Waiting for calls...\n");
            
            }
            catch (Exception e)
            {
                trace.appendText( "Exception caught - Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
        }
    }

    public void stop()
    {
        if ((myRouteAddress != null) && (myRouteCallback != null))
        {
            // cancel previous registration to route calls for
            try
            {
                // myRouteAddress
                myRouteAddress.cancelRouteCallback(myRouteCallback);
                trace.appendText("Cancelled Route Callback for " + myRouteAddress.getName() + "\n");
            }
            catch (Exception e)
            {
                trace.appendText( "Exception caught - Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
        }
    }
        
    public void paint( Graphics d ) {
    	Dimension size;
    	
    	size = size();
    	trace.resize( size.width, size.height - 20);
    	return;
    }
}

class MyRouteCallback implements RouteCallback
{
        MyRouteCallback() {
        }
        
        public MyRouteCallback(String _destination, MyTextArea traceArea)
        {
            this();
            destination = _destination;
            trace = traceArea;
        }
        
        public void routeEvent(RouteEvent event)
        {
            trace.appendText("Received Route Event...\n");
            trace.appendText("\tcallingAddress=" + event.getCallingAddress().getName() + "\n");
            trace.appendText("\tcallingTerminal=" + event.getCallingTerminal().getName() + "\n");
            trace.appendText("\tcurrentRouteAddress=" + event.getCurrentRouteAddress().getName() + "\n");
            trace.appendText("\trouteSelectAlgorithm=" + event.getRouteSelectAlgorithm() + "\n");
            trace.appendText("\tsetupInformation=" + event.getSetupInformation() + "\n");
            //trace.appendText("\trouteSession=" + event.getRouteSession() + "\n");
            // call function todetermine a destination
            String[] routeSelected = new String[1];
            routeSelected[0] = destination;
            try
            {
                trace.appendText("\tcause=" + event.getRouteSession().getCause() + "\n");
                trace.appendText("\trouteAddress=" + event.getRouteSession().getRouteAddress().getName() + "\n");
                trace.appendText("\tstate=" + event.getRouteSession().getState() + "\n");
                trace.appendText("Selected route: " + routeSelected[0] + "\n");
                event.getRouteSession().selectRoute(routeSelected);
                //event.getRouteSession().endRoute(RouteSession.CAUSE_NO_ERROR);
                //event.getCurrentRouteAddress().cancelRouteCallback(this);
                RouteSession[] activeSessions = event.getCurrentRouteAddress().getActiveRouteSessions();
                if (activeSessions != null) {
                    for(int i=0; i<activeSessions.length; i++)
                    {
                        trace.appendText("activeSessions[" + i + "] state=" + activeSessions[i].getState() + "\n");
                    }
                } else {
                    trace.appendText("activeSessions is null\n");
                }
                
            }
            catch (Exception e)
            {
                trace.appendText( "Exception caught - Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            return;
        }

        public void reRouteEvent(ReRouteEvent event)
        {
            trace.appendText("Received reRoute Event\n");
            trace.appendText("\trouteSession=" + event.getRouteSession() + "\n");
        }

        public void routeUsedEvent(RouteUsedEvent event)
        {
            trace.appendText("Received Route Used  Event\n");
            trace.appendText("\tcallingAddress=" + event.getCallingAddress().getName() + "\n");
            trace.appendText("\tcallingTerminal=" + event.getCallingTerminal().getName() + "\n");
            trace.appendText("\tdomain=" + event.getDomain() + "\n");
            trace.appendText("\trouteUsed=" + event.getRouteUsed().getName() + "\n");
        }

        public void routeEndEvent(RouteEndEvent event)
        {
            trace.appendText("Received Route End  Event\n");
            RouteSession activeSession = event.getRouteSession();
            if (activeSession != null) {
                    trace.appendText("activeSession state=" + activeSession.getState() + "\n");
            } else {
                trace.appendText("activeSession is null\n");
            }
        }

        public void routeCallbackEndedEvent(RouteCallbackEndedEvent event)
        {
            trace.appendText("Received Route Callback Ended Event\n");
            trace.appendText("\trouteAddress=" + event.getRouteAddress().getName() + "\n");
        }


        private String      destination;
        private MyTextArea  trace;
}